/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.test.aspects.remoting;

import java.util.ArrayList;
import java.util.List;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.metadata.SimpleMetaData;

/**
 * An interceptor that comes next in the chain after ClusterChooserInterceptor.
 * 
 * @author Brian Stansberry
 */
public class MockNextInterceptor implements Interceptor
{
   private List<SimpleMetaData> invocationMetaData = new ArrayList<SimpleMetaData>();
   private Object[] returnValues;
   private int idx;
   
   public String getName()
   {
      return "MockNextInterceptor";
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      SimpleMetaData latest = new SimpleMetaData();
      latest.mergeIn(invocation.getMetaData());
      invocationMetaData.add(latest);
      Object result = null;
      if (returnValues != null)
      {
         try
         {
            if (returnValues[idx] instanceof Throwable)
            {
               throw (Throwable) returnValues[idx];
            }
            result = returnValues[idx];
         }
         finally
         {
            idx++;
         }
      }
      return result;
   }

   /**
    * Gets a list containing copies of the SimpleMetaData that has been
    * passed to each invocation of invoke().
    * 
    * @return
    */
   public List<SimpleMetaData> getInvocationHistory()
   {
      return invocationMetaData;
   }

   /** 
    * Sets a sequence of return values that should be returned from invoke().
    * Typically this would be one per expected invocation during a series
    * of failover attempts.
    */
   public void setReturnValues(Object[] returnValues)
   {
      this.returnValues = returnValues;
   }

   /** Convenience method to set a single return value for a non-failover test */
   public void setReturnValue(Object returnValue)
   {
      if (returnValue != null)
      {
         returnValues = new Object[] { returnValue };
      }
      else
      {
         returnValues = null;
      }
   }
}
