/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.test.ha.client.loadbalance;

import java.util.Arrays;

import org.jboss.ha.client.loadbalance.FirstAvailableIdenticalAllProxies;
import org.jboss.ha.client.loadbalance.LoadBalancePolicy;
import org.jboss.ha.framework.interfaces.ClusteringTargetsRepository;
import org.jboss.ha.framework.interfaces.FamilyClusterInfo;

/**
 * Tests of the FirstAvailable load balance policy.
 * 
 * @author Brian Stansberry
 *
 */
public class FirstAvailableIdenticalAllProxiesUnitTestCase extends FirstAvailableUnitTestCase
{

   /**
    * Create a new FirstAvailableIdenticalAllProxiesUnitTestCase.
    * 
    * @param name
    */
   public FirstAvailableIdenticalAllProxiesUnitTestCase(String name)
   {
      super(name);
   }
   
   

   @Override
   protected LoadBalancePolicy getLoadBalancePolicy()
   {
      return new FirstAvailableIdenticalAllProxies();
   }

   @Override
   public void testInitialSelection()
   {
      testCount++;
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, Arrays.asList(TARGETS));
      
      LoadBalancePolicy lbp = getLoadBalancePolicy();
      Object target = lbp.chooseTarget(fci);
      
      for (int i = 0; i < 10000; i++)
      {
         assertSame(target, getLoadBalancePolicy().chooseTarget(fci));
      }
      
   }
   
   public void testIdenticalAfterFailover()
   {
      testCount++;
      FamilyClusterInfo fci = ClusteringTargetsRepository.initTarget(FAMILY_BASE + testCount, Arrays.asList(TARGETS));
      
      LoadBalancePolicy lbp1 = getLoadBalancePolicy();
      Object target1 = lbp1.chooseTarget(fci);
      
      LoadBalancePolicy lbp2 = getLoadBalancePolicy();
      
      assertSame(target1, lbp2.chooseTarget(fci));
      
      fci.removeDeadTarget(target1);
      
      Object target3 = lbp1.chooseTarget(fci);
      
      assertFalse(target1.equals(target3));
      
      assertSame(target3, lbp2.chooseTarget(fci));
      
   }

   
}
